@@ -3,7 +3,7 @@
 #
 # graph_tool -- a general graph manipulation python module
 #
-# Copyright (C) 2006-2021 Tiago de Paula Peixoto <tiago@skewed.de>
+# Copyright (C) 2006-2025 Tiago de Paula Peixoto <tiago@skewed.de>
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU Lesser General Public License as published by the Free
@@ -80,7 +80,7 @@ def modularity(g, b, gamma=1., weight=None):
        :doi:`10.1073/pnas.0601602103`, :arxiv:`physics/0602124`
     """
 
-    if b.value_type() not in ["bool", "int16_t", "int32_t", "int64_t",
+    if b.value_type() not in ["bool", "int16_t", "int64_t", "int64_t",
                               "unsigned long"]:
         b = perfect_prop_hash([b])[0]
     Q = libinference.modularity(g._Graph__graph, gamma,
@@ -89,19 +89,30 @@ def modularity(g, b, gamma=1., weight=None):
     return Q
 
 
+@entropy_state_signature
 class ModularityState(MCMCState, MultiflipMCMCState, MultilevelMCMCState,
-                      GibbsMCMCState):
-    r"""Obtain the partition of a network according to Newman's modularity.
+                      GibbsMCMCState, DrawBlockState):
+    r"""Obtain the partition of a network according to the maximization of Newman's modularity.
 
-    .. warning::
+    .. danger:: Using modularity maximization is almost always **a terrible idea**.
+
+       Modularity maximization is a substantially inferior method to the
+       inference-based ones that are implemented in ``graph-tool``, since it
+       does not possess any kind of statistical regularization. Among many other
+       problems, the method tends to massively overfit empirical data.
+
+       For a more detailed explanation see `“Modularity maximization
+       considered harmful”
+       <https://skewed.de/tiago/blog/modularity-harmful>`_, as well as
+       [peixoto-descriptive-2023]_.
 
        Do not use this approach in the analysis of networks without
        understanding the consequences. This algorithm is included only for
-       comparison purposes. In general, the inference-based approaches based on
-       :class:`~graph_tool.inference.blockmodel.BlockState`,
-       :class:`~graph_tool.inference.nested_blockmodel.NestedBlockState`, and
-       :class:`~graph_tool.inference.planted_partition.PPBlockState` should be
-       universally preferred.
+       comparison purposes. In general, the inference-based approaches based
+       on :class:`~graph_tool.inference.BlockState`,
+       :class:`~graph_tool.inference.NestedBlockState`, and
+       :class:`~graph_tool.inference.PPBlockState` should be universally
+       preferred.
 
     Parameters
     ----------
@@ -110,44 +121,45 @@ class ModularityState(MCMCState, MultiflipMCMCState, MultilevelMCMCState,
     b : :class:`~graph_tool.PropertyMap` (optional, default: ``None``)
         Initial partition. If not supplied, a partition into a single group will
         be used.
-
+    eweight : :class:`~graph_tool.EdgePropertyMap` (optional, default: ``None``)
+        Edge multiplicities (for multigraphs).
+    entropy_args: ``dict`` (optional, default: ``{}``)
+        Override default arguments for :meth:`~ModularityState.entropy()`
+        method and releated operations.
     """
 
-    def __init__(self, g, b=None):
-
+    def __init__(self, g, b=None, eweight=None, entropy_args={}):
+        EntropyState.__init__(self, entropy_args=entropy_args)
         self.g = GraphView(g, directed=False)
         if b is None:
-            self.b = self.g.new_vp("int32_t")
-        elif isinstance(g, PropertyMap):
-            self.b = b.copy("int32_t")
+            self.b = self.g.new_vp("int64_t")
+        elif isinstance(b, PropertyMap):
+            self.b = self.g.own_property(b).copy("int64_t")
         else:
-            self.b = self.g.new_vp("int32_t", vals=b)
+            self.b = self.g.new_vp("int64_t", vals=b)
 
-        self.er = Vector_size_t()
-        self.err = Vector_size_t()
+        if eweight is None:
+            eweight = g.new_ep("int64_t", 1)
+        else:
+            eweight = g.own_property(eweight)
+        self.eweight = eweight
 
-        self.bg = self.g
+        self.bg = Graph(g.num_vertices(), directed=False)
         self._abg = self.bg._get_any()
         self._state = libinference.make_modularity_state(self)
 
-        self._entropy_args = dict(gamma=1.)
-
     def __copy__(self):
         return self.copy()
 
-    def __deepcopy__(self, memo):
-        g = copy.deepcopy(self.g, memo)
-        b = copy.deepcopy(self.b, memo)
-        return self.copy(g=g, b=b)
-
     def copy(self, g=None, b=None):
         r"""Copies the state. The parameters override the state properties, and
          have the same meaning as in the constructor."""
         return ModularityState(g=g if g is not None else self.g,
-                                b=b if b is not None else self.b)
+                               b=b if b is not None else self.b)
 
     def __getstate__(self):
-        return dict(g=self.g, b=self.b)
+        state = EntropyState.__getstate__(self)
+        return dict(state, g=self.g, b=self.b)
 
     def __setstate__(self, state):
         self.__init__(**state)
@@ -156,22 +168,27 @@ class ModularityState(MCMCState, MultiflipMCMCState, MultilevelMCMCState,
         return "<ModularityState object with %d blocks, for graph %s, at 0x%x>" % \
             (self.get_B(), str(self.g), id(self))
 
+    def _gen_eargs(self, args):
+        return libinference.modularity_entropy_args()
+
     def get_B(self):
         r"Returns the total number of blocks."
-        return len(np.unique(self.b.fa))
+        rs = np.unique(self.b.fa)
+        return len(rs)
 
     def get_Be(self):
         r"""Returns the effective number of blocks, defined as :math:`e^{H}`, with
         :math:`H=-\sum_r\frac{n_r}{N}\ln \frac{n_r}{N}`, where :math:`n_r` is
         the number of nodes in group r.
         """
-        w = np.array(np.bincount(self.b.fa), dtype="double")
+        w = np.bincount(self.b.fa)
+        w = np.array(w, dtype="double")
         w = w[w>0]
         w /= w.sum()
-        return np.exp(-(w*log(w)).sum())
+        return np.exp(-(w*np.log(w)).sum())
 
     @copy_state_wrap
-    def entropy(self, gamma=1., **kwargs):
+    def _entropy(self, gamma=1.):
         r"""Return the unnormalized negative generalized modularity.
 
         Notes
@@ -190,10 +207,6 @@ class ModularityState(MCMCState, MultiflipMCMCState, MultilevelMCMCState,
 
         eargs = self._get_entropy_args(locals(), ignore=["self", "kwargs"])
 
-        if len(kwargs) > 0:
-            raise ValueError("unrecognized keyword arguments: " +
-                             str(list(kwargs.keys())))
-
         return self._state.entropy(eargs)
 
     def _get_entropy_args(self, kwargs, ignore=None):
@@ -229,22 +242,6 @@ class ModularityState(MCMCState, MultiflipMCMCState, MultilevelMCMCState,
         Q = self.entropy(gamma=gamma)
         return -Q / (2 * self.g.num_edges())
 
-    def draw(self, **kwargs):
-        r"""Convenience wrapper to :func:`~graph_tool.draw.graph_draw` that
-        draws the state of the graph as colors on the vertices and edges."""
-        gradient = self.g.new_ep("double")
-        gradient = group_vector_property([gradient])
-        from graph_tool.draw import graph_draw
-        return graph_draw(self.g,
-                          vertex_fill_color=kwargs.get("vertex_fill_color",
-                                                       self.b),
-                          vertex_color=kwargs.get("vertex_color", self.b),
-                          edge_gradient=kwargs.get("edge_gradient",
-                                                   gradient),
-                          **dmask(kwargs, ["vertex_fill_color",
-                                           "vertex_color",
-                                           "edge_gradient"]))
-
     def _mcmc_sweep_dispatch(self, mcmc_state):
         return libinference.modularity_mcmc_sweep(mcmc_state, self._state,
                                                   _get_rng())
